import dsc40graph
from DisjointForest import DisjointSetForest


def dfs(graph, u, status=None):
    """Start a DFS at `u`."""
    # initialize status if it was not passed
    if status is None:
        status = {node: 'undiscovered' for node in graph.nodes}
        
    status[u] = 'pending'
    for v in graph.neighbors(u): # explore edge (u, v)
        if status[v] == 'undiscovered':
            dfs(graph, v, status)
    status[u] = 'visited'

def dfs_set(graph, u, s, status=None):
    """Start a DFS at `u`."""
    # initialize status if it was not passed
    if status is None:
        status = {node: 'undiscovered' for node in graph.nodes}
    
    status[u] = 'pending'
    for v in graph.neighbors(u): # explore edge (u, v)
        if status[v] == 'undiscovered':
            s.append(v)
            dfs_set(graph, v, s, status)
    status[u] = 'visited'

def NumberOfconnectedComponents(graph):
    status = {node: 'undiscovered' for node in graph.nodes}
    count = 0
    for node in graph.nodes:
        if status[node] == 'undiscovered':
            dfs(graph, node, status)
            count += 1
    return count

def froset(graph):
    status = {node: 'undiscovered' for node in graph.nodes}
    l = []
    for node in graph.nodes:
        if status[node] == 'undiscovered':
            s = [node]
            dfs_set(graph, node, s, status)
            l.append(set(s))
    return l

def slc(graph, d, k):
    clus = dsc40graph.UndirectedGraph()
    for node in graph.nodes:
        clus.add_node(node)
    # place each node in its own disjoint set
    components = DisjointSetForest(graph.nodes)
    
    # sorted edges in ascending order by weight
    sorted_edges = sorted(list(graph.edges), key=d)
    
    for (u, v) in sorted_edges:
        if not components.in_same_set(u, v):
            # If number of clusters = k, Break
            if NumberOfconnectedComponents(clus) != k:
                clus.add_edge(u, v)
                components.union(u, v)
            else:
                break
            
    f = froset(clus) 
    l = []
    for set in f:
        l.append(frozenset(set))

    return frozenset(l)

